//Keep all basic weapon replacement and standalone implementations here
class LCMutator expands XC_LagCompensation;

var Weapon ReplaceThis, ReplaceWith;
var LCSpawnNotify ReplaceSN;
var bool bApplySNReplace;

//Find known custom arenas, replace with LC arenas
event PreBeginPlay()
{
	local Mutator M, old;

	Super.PreBeginPlay(); //XC_LagCompensation will hook the monsters

	//New arena is hooked right before this LCMutator
	if ( FoundArena( Level.Game.BaseMutator) )
	{
		M = Level.Game.BaseMutator;
		Level.Game.BaseMutator = M.nextMutator;
		M.Destroy();
	}
	old = Level.Game.BaseMutator;
	For ( M=old.NextMutator ; M!=none ; M=M.nextMutator )
	{
		if ( FoundArena( M) )
		{
			old.NextMutator = M.NextMutator;
			M.Destroy();
		}
		else
			old = M;
	}
}

function AddMutator(Mutator M)
{
	//Do not add this arena if it can be replaced
	if ( FoundArena(M) || M==self )
	{
		return;
	}

	if ( NextMutator == None )
		NextMutator = M;
	else
		NextMutator.AddMutator(M);
}

event PostBeginPlay()
{
	ReplaceSN = Spawn(class'LCSpawnNotify');
	ReplaceSN.Mutator = self;
}

function ModifyPlayer( Pawn Other)
{
	local XC_LagCompensator XCLC;
	Super.ModifyPlayer(Other);

	if ( ffFindCompFor(Other) == none )
		ffInsertNewPlayer( Other);
}

function bool CheckReplacement (Actor Other, out byte bSuperRelevant)
{
	if ( Weapon(Other) == none )
		return true;
//		return Super.CheckReplacement(Other, bSuperRelevant); //Call XC_LagCompensation version

	if ( ClassIsChildOf( Other.Class, class'Enforcer') )
	{
		if ( Other.Class == class'Enforcer' )
			return DoReplace(Weapon(Other),class'LCEnforcer');
		else if ( Other.IsA('sgEnforcer') )
			return DoReplace(Weapon(Other),class'LCEnforcer',,true);
	}
	else if ( ClassIsChildOf( Other.Class, class'ShockRifle') )
	{
		if ( Other.Class == class'ShockRifle' )
			return DoReplace(Weapon(Other),class'LCShockRifle');
		if ( Other.Class == class'SuperShockRifle' )
			return DoReplace(Weapon(Other),class'LCSuperShockRifle');
		if ( Other.IsA('AdvancedShockRifle') )
			return DoReplace(Weapon(Other),class'LCAdvancedShockRifle');
	}
	else if ( ClassIsChildOf( Other.Class, class'SniperRifle') )
	{
		if ( Other.Class == class'SniperRifle' )
			return DoReplace(Weapon(Other),class'LCSniperRifle');
		else if ( Other.IsA('SniperRifle2x') ) //AWM_Beta1 rifle
			return DoReplace(Weapon(Other),class'LCSniperRifle',,true);
	}
	else if ( ClassIsChildOf( Other.Class, class'minigun2') )
	{
		if ( Other.Class == class'minigun2' )
			return DoReplace(Weapon(Other),class'LCMinigun2');
		else if ( Other.IsA('Minigun_2x') )
			return DoReplace(Weapon(Other),class'LCMinigun2',,true);
		else if ( Other.IsA('sgMinigun') )
			return DoReplace(Weapon(Other),class'LCMinigun2',,true);
	}
	else if ( Other.IsA('LMinigun') ) //Liandri minigun
	{
		Class'LCLiandriMinigun'.default.OrgClass = class<TournamentWeapon>(Other.Class);
		return DoReplace( Weapon(Other), class'LCLiandriMinigun');
	}
	else if ( Other.IsA('AsmdPulseRifle') ) //SiegeXtreme
	{
		Class'LCAsmdPulseRifle'.default.OrgClass = class<TournamentWeapon>(Other.Class);
		return DoReplace( Weapon(Other), class'LCAsmdPulseRifle');
	}
	else if ( Other.IsA('SiegeInstagibRifle') ) //SiegeUltimate
	{
		Class'LCSiegeInstagibRifle'.default.OrgClass = class<TournamentWeapon>(Other.Class);
		return DoReplace( Weapon(Other), class'LCSiegeInstagibRifle');
	}

	return True;
}

function bool DoReplace( Weapon Other, class<Weapon> NewWeapClass, optional bool bFullAmmo, optional bool bCopyAmmo)
{
	local Weapon W;

//	if ( Other.Location == vect(0,0,0) ) //Replace differently
//		return true;

	W = Other.Spawn(NewWeapClass);
	if ( W != none )
	{
		W.SetCollisionSize( Other.CollisionRadius, Other.CollisionHeight);
		W.Tag = Other.Tag;
		W.Event = Other.Event;
		if ( Other.MyMarker != none )
		{
			W.MyMarker = Other.MyMarker;
			W.MyMarker.markedItem = W;
		}
		W.bHeldItem = Other.bHeldItem;
		W.RespawnTime = Other.RespawnTime;
		W.PickupAmmoCount = Other.PickupAmmoCount;
		if ( bCopyAmmo )
			W.AmmoName = Other.AmmoName;
		if ( bFullAmmo )
			W.PickupAmmoCount = W.AmmoName.default.MaxAmmo;
		W.bRotatingPickup = Other.bRotatingPickup;
		SetReplace( Other, W);
		return bApplySNReplace;
	}
	return true;
}

function SetReplace( Weapon Other, Weapon With)
{
	ReplaceThis = Other;
	ReplaceWith = With;
	if ( ReplaceThis != none && ReplaceWith != none)
		ReplaceSN.ActorClass = ReplaceThis.class;
	else
		ReplaceSN.ActorClass = class'LCDummyWeapon';
}


//***************************************************
//This function is massive, deal with each known case
//***************************************************
function bool FoundArena( mutator M)
{
	local LCArenaMutator LCArena;

	if ( Arena(M) != none )
	{
		if ( M.IsA('SniperArena') )
		{
			LCArena = Spawn( class'LCArenaMutator');
			LCArena.SetupWeaponReplace( class'SniperRifle', class'LCSniperRifle');
		}
		else if ( M.IsA('ShockArena') )
		{
			LCArena = Spawn( class'LCArenaMutator');
			LCArena.SetupWeaponReplace( class'ShockRifle', class'LCShockRifle');
		}
		else if ( M.IsA('InstaGibDM') )
		{
			LCArena = Spawn( class'LCArenaMutator');
			LCArena.SetupWeaponReplace( class'SuperShockRifle', class'LCSuperShockRifle');
			if ( !ChainMutatorBeforeThis(LCArena) )
				return false;
			LCArena.LCMutator = self;
			LCArena.SetupWeaponRespawn( true, true, true, true);
			LCArena.SetupPickups( true, true, false, true);
			return true;
		}

		if ( LCArena != none )
		{
			if ( !ChainMutatorBeforeThis(LCArena) )
				return false;
			LCArena.LCMutator = self;
			LCArena.SetupWeaponRespawn( true, true, true, true);
			LCArena.SetupPickups( false, false, false, true);
			return true;
		}
	}
	else if ( M.IsA('NYACovertSniper_RIFLEMutator') )
	{
		LCArena = Spawn( class'LCArenaMutator');
		LCArena.SetupWeaponReplace( class<Weapon>(DynamicLoadObject("NYACovertSniper.NYACovertSniper",class'class')), class'LCNYACovertSniper');
		if ( !ChainMutatorBeforeThis(LCArena) )
			return false;
		LCArena.LCMutator = self;
		LCArena.SetupWeaponRespawn( true, true, true, true);
		LCArena.SetupPickups( false, false, false, true);
		return true;
	}

	
}

function bool ChainMutatorBeforeThis( Mutator M)
{
	local Mutator MU;
	M.NextMutator = self;
	if ( Level.Game.BaseMutator == self )
	{
		Level.Game.BaseMutator = M;
		return true;
	}

	For ( MU=Level.Game.BaseMutator ; MU.NextMutator != none ; MU=MU.NextMutator )
	{
		if ( MU.NextMutator == self )
		{
			MU.NextMutator = M;
			return true;
		}
	}

	Level.Game.BaseMutator.AddMutator( M);
	return true;
}


//*******************************
//******************* MUTATE

//Mimicking ZP because ppl gets used to stuff
function Mutate (string MutateString, PlayerPawn Sender)
{
	if ( (MutateString ~= "getweapon zp_SniperRifle") || (MutateString ~= "getweapon zp_sn") )
		Class'LCStatics'.static.FindBasedWeapon( Sender, class'LCSniperRifle');
	else if ( (MutateString ~= "getweapon zp_ShockRifle") || (MutateString ~= "getweapon zp_sh") )
		Class'LCStatics'.static.FindBasedWeapon( Sender, class'LCShockRifle');
	else if ( (MutateString ~= "getweapon zp_Enforcer") || (MutateString ~= "getweapon zp_e") )
		Class'LCStatics'.static.FindBasedWeapon( Sender, class'LCEnforcer');
	else if ( (MutateString ~= "getweapon lc_apr") )
		Class'LCStatics'.static.FindBasedWeapon( Sender, class'LCAsmdPulseRifle');
	else if ( (MutateString ~= "getweapon lc_sir") )
		Class'LCStatics'.static.FindBasedWeapon( Sender, class'LCSiegeInstagibRifle');
	else if ( (MutateString ~= "getweapon lc_m") )
		Class'LCStatics'.static.FindBasedWeapon( Sender, class'LCMinigun2');
	else if ( MutateString ~= "zp_Off" )
	{
		Sender.ClientMessage("Zeroping disabled.");
		Sender.ClientMessage("Type 'mutate zp_on' to restore.");
		ffFindCompFor(Sender).CompChannel.ClientChangeLC(false);
	}
	else if ( MutateString ~= "zp_On" )
	{
		Sender.ClientMessage("Zeroping enabled.");
		Sender.ClientMessage("Type 'mutate zp_off' to disable.");
		ffFindCompFor(Sender).CompChannel.ClientChangeLC(true);
	}
	else if ( MutateString ~= "state" )
		Sender.ClientMessage("Weapon State:" @ string(Sender.Weapon.GetStateName()));
	Super.Mutate(MutateString,Sender);
}